@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf8 -*-
 # ============================================================================
-#  Copyright (c) 2013-2017 nexB Inc. http://www.nexb.com/ - All rights reserved.
+#  Copyright (c) 2013-2018 nexB Inc. http://www.nexb.com/ - All rights reserved.
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
@@ -34,21 +34,26 @@ import json
 import os
 import posixpath
 from posixpath import dirname
+
+import yaml
 import re
 import sys
 
 if sys.version_info[0] < 3:  # Python 2
-    import backports.csv as csv
-    from urlparse import urljoin, urlparse
-    from urllib2 import urlopen, Request, HTTPError
+    import backports.csv as csv  # NOQA
+    from itertools import izip_longest as zip_longest  # NOQA
+    from urlparse import urljoin, urlparse  # NOQA
+    from urllib2 import urlopen, Request, HTTPError  # NOQA
 else:  # Python 3
-    basestring = str
-    import csv
-    from urllib.parse import urljoin, urlparse
-    from urllib.request import urlopen, Request
-    from urllib.error import HTTPError
+    basestring = str  # NOQA
+    import csv  # NOQA
+    from itertools import zip_longest  # NOQA
+    from urllib.parse import urljoin, urlparse  # NOQA
+    from urllib.request import urlopen, Request  # NOQA
+    from urllib.error import HTTPError  # NOQA
 
 from license_expression import Licensing
+import saneyaml
 
 from attributecode import CRITICAL
 from attributecode import ERROR
@@ -56,13 +61,12 @@ from attributecode import INFO
 from attributecode import WARNING
 from attributecode import api
 from attributecode import Error
-from attributecode import saneyaml
 from attributecode import util
 from attributecode.util import add_unc
 from attributecode.util import copy_license_notice_files
 from attributecode.util import on_windows
+from attributecode.util import ungroup_licenses
 from attributecode.util import UNC_PREFIX
-from attributecode.util import UNC_PREFIX_POSIX
 
 
 class Field(object):
@@ -70,6 +74,7 @@ class Field(object):
     An ABOUT file field. The initial value is a string. Subclasses can and
     will alter the value type as needed.
     """
+
     def __init__(self, name=None, value=None, required=False, present=False):
         # normalized names are lowercased per specification
         self.name = name
@@ -106,13 +111,10 @@ class Field(object):
                 msg = u'Field %(name)s is required'
                 errors.append(Error(CRITICAL, msg % locals()))
                 return errors
-            """else:
-                # no error for not present non required fields
-                # FIXME: should we add an info?
-                # CY: I don't think so.
-                pass"""
         else:
             # present fields should have content ...
+            # The boolean value can be True, False and None
+            # The value True or False is the content of boolean fields
             if not self.has_content:
                 # ... especially if required
                 if self.required:
@@ -175,7 +177,16 @@ class Field(object):
                     # insert 4 spaces for newline values
                     value = u'    '.join(value)
             else:
-                value = u''.join(value)
+                # See https://github.com/nexB/aboutcode-toolkit/issues/323
+                # The yaml.load() will throw error if the parsed value
+                # contains ': ' character. A work around is to put a pipe, '|'
+                # to indicate the whole value as a string
+                if value and ': ' in value[0]:
+                    value.insert(0, u'|\n')
+                    # insert 4 spaces for newline values
+                    value = u'    '.join(value)
+                else:
+                    value = u''.join(value)
 
         serialized = u'%(name)s:' % locals()
         if value:
@@ -281,6 +292,8 @@ class ListField(StringField):
 
         if isinstance(self.original_value, basestring):
             values = self.original_value.splitlines(False)
+        elif isinstance(self.original_value, list):
+            values = self.original_value
         else:
             values = [repr(self.original_value)]
 
@@ -304,7 +317,7 @@ class ListField(StringField):
         return errors
 
     def _serialized_value(self):
-        return u'\n'.join(self.value) if self.value else u''
+        return self.value if self.value else u''
 
     def __eq__(self, other):
         """
@@ -330,8 +343,7 @@ class ListField(StringField):
         if sval == oval:
             return True
 
-
-class UrlField(ListField):
+class UrlListField(ListField):
     """
     A URL field. The validated value is a list of URLs.
     """
@@ -339,11 +351,11 @@ class UrlField(ListField):
         """
         Check that URLs are valid. Return a list of errors.
         """
-        errors = super(UrlField, self)._validate(*args, ** kwargs)
-        for url in self.value:
+        errors = super(UrlListField, self)._validate(*args, ** kwargs)
+        name = self.name
+        val = self.value
+        for url in val:
             if not self.is_valid_url(url):
-                name = self.name
-                val = self.value
                 msg = (u'Field %(name)s: Invalid URL: %(val)s' % locals())
                 errors.append(Error(WARNING, msg))
         return errors
@@ -358,6 +370,32 @@ class UrlField(ListField):
         return valid
 
 
+class UrlField(StringField):
+    """
+    A URL field. The validated value is a URL.
+    """
+    def _validate(self, *args, **kwargs):
+        """
+        Check that URL is valid. Return a list of errors.
+        """
+        errors = super(UrlField, self)._validate(*args, ** kwargs)
+        name = self.name
+        val = self.value
+        if not self.is_valid_url(val):
+            msg = (u'Field %(name)s: Invalid URL: %(val)s' % locals())
+            errors.append(Error(WARNING, msg))
+        return errors
+
+    @staticmethod
+    def is_valid_url(url):
+        """
+        Return True if a URL is valid.
+        """
+        scheme, netloc, _path, _p, _q, _frg = urlparse(url)
+        valid = scheme in ('http', 'https', 'ftp') and netloc
+        return valid
+
+
 class PathField(ListField):
     """
     A field pointing to one or more paths relative to the ABOUT file location.
@@ -385,10 +423,8 @@ class PathField(ListField):
             self.base_dir = util.to_posix(self.base_dir)
 
         name = self.name
-        # FIXME: This is a temp fix for #286
-        # The field in ignore_checking_list is validated in the check_file_field_exist function
-        ignore_checking_list = [u'license_file', u'notice_file', u'changelog_file']
 
+        # FIXME: Why is the PathField an ordered dict?
         # mapping of normalized paths to a location or None
         paths = OrderedDict()
 
@@ -416,18 +452,18 @@ class PathField(ListField):
                 paths[path] = location
                 continue
 
-            if self.license_notice_text_location and name in ignore_checking_list:
+            if self.license_notice_text_location:
                 location = posixpath.join(self.license_notice_text_location, path)
             else:
-                # The 'about_resource_path' should be a joined path with
+                # The 'about_resource' should be a joined path with
                 # the 'about_file_path' and the 'base_dir
                 if not self.running_inventory and self.about_file_path:
                     # Get the parent directory of the 'about_file_path'
                     afp_parent = posixpath.dirname(self.about_file_path)
 
-                    # Create a relative 'about_resource_path' by joining the
+                    # Create a relative 'about_resource' path by joining the
                     # parent of the 'about_file_path' with the value of the
-                    # 'about_resource_path'
+                    # 'about_resource'
                     arp = posixpath.join(afp_parent, path)
                     normalized_arp = posixpath.normpath(arp).strip(posixpath.sep)
                     location = posixpath.join(self.base_dir, normalized_arp)
@@ -442,9 +478,12 @@ class PathField(ListField):
             if not os.path.exists(location):
                 # We don't want to show the UNC_PREFIX in the error message
                 location = util.to_posix(location.strip(UNC_PREFIX))
-                if not name in ignore_checking_list:
-                    msg = (u'Field %(name)s: Path %(location)s not found'
-                           % locals())
+                msg = (u'Field %(name)s: Path %(location)s not found'
+                       % locals())
+                # We want to show INFO error for 'about_resource'
+                if name == u'about_resource':
+                    errors.append(Error(INFO, msg))
+                else:
                     errors.append(Error(CRITICAL, msg))
                 location = None
 
@@ -535,9 +574,9 @@ class BooleanField(SingleLineField):
     def default_value(self):
         return None
 
-    flags = {'yes': True, 'y': True, 'no': False, 'n': False, }
-
-    flag_values = ', '.join(flags)
+    true_flags = ('yes', 'y', 'true', 'x')
+    false_flags = ('no', 'n', 'false')
+    flag_values = true_flags + false_flags
 
     def _validate(self, *args, **kwargs):
         """
@@ -545,13 +584,14 @@ class BooleanField(SingleLineField):
         False. Return a list of errors.
         """
         errors = super(BooleanField, self)._validate(*args, ** kwargs)
-
+        self.about_file_path = kwargs.get('about_file_path')
         flag = self.get_flag(self.original_value)
         if flag is False:
             name = self.name
             val = self.original_value
+            about_file_path = self.about_file_path
             flag_values = self.flag_values
-            msg = (u'Field %(name)s: Invalid flag value: %(val)r is not '
+            msg = (u'Path: %(about_file_path)s - Field %(name)s: Invalid flag value: %(val)r is not '
                    u'one of: %(flag_values)s' % locals())
             errors.append(Error(ERROR, msg))
             self.value = None
@@ -562,13 +602,17 @@ class BooleanField(SingleLineField):
             errors.append(Error(INFO, msg))
             self.value = None
         else:
-            self.value = self.flags.get(flag)
+            if flag == u'yes' or flag is True:
+                self.value = True
+            else:
+                self.value = False
         return errors
 
     def get_flag(self, value):
         """
         Return a normalized existing flag value if found in the list of
-        possible values or None if empty or False if not found.
+        possible values or None if empty or False if not found or original value
+        if it is not a boolean value
         """
         if value is None or value == '':
             return None
@@ -583,8 +627,10 @@ class BooleanField(SingleLineField):
 
                 value = value.lower()
                 if value in self.flag_values:
-                    # of of yes, no, true, etc.
-                    return value
+                    if value in self.true_flags:
+                        return u'yes'
+                    else:
+                        return u'no'
                 else:
                     return False
             else:
@@ -592,12 +638,12 @@ class BooleanField(SingleLineField):
 
     @property
     def has_content(self):
-        # Special for flags: None means False AND content not defined
-        flag = self.get_flag(self.original_value)
-        if flag:
+        """
+        Return true if it has content regardless of what value, False otherwise
+        """
+        if self.original_value:
             return True
-        else:
-            return flag
+        return False
 
     def _serialized_value(self):
         # default normalized values for serialization
@@ -619,7 +665,7 @@ class BooleanField(SingleLineField):
                 and self.value == other.value)
 
 
-def validate_fields(fields, about_file_path, running_inventory, base_dir, 
+def validate_fields(fields, about_file_path, running_inventory, base_dir,
                     license_notice_text_location=None):
     """
     Validate a sequence of Field objects. Return a list of errors.
@@ -660,10 +706,10 @@ class About(object):
         is simpler.
         """
         self.fields = OrderedDict([
-            ('about_resource', ListField(required=True)),
+            # ('about_resource', ListField(required=True)),
             # ('about_resource', AboutResourceField(required=True)),
+            ('about_resource', AboutResourceField(required=True)),
             ('name', SingleLineField(required=True)),
-            ('about_resource_path', AboutResourceField()),
 
             ('version', SingleLineField()),
             ('download_url', UrlField()),
@@ -671,26 +717,28 @@ class About(object):
             ('homepage_url', UrlField()),
             ('notes', StringField()),
 
-            ('license', ListField()),
             ('license_expression', StringField()),
+            ('license_key', ListField()),
             ('license_name', ListField()),
             ('license_file', FileTextField()),
-            ('license_url', UrlField()),
+            ('license_url', UrlListField()),
             ('copyright', StringField()),
             ('notice_file', FileTextField()),
             ('notice_url', UrlField()),
 
             ('redistribute', BooleanField()),
             ('attribute', BooleanField()),
-            ('track_change', BooleanField()),
+            ('track_changes', BooleanField()),
             ('modified', BooleanField()),
+            ('internal_use_only', BooleanField()),
 
             ('changelog_file', FileTextField()),
 
-            ('owner', ListField()),
+            ('owner', StringField()),
             ('owner_url', UrlField()),
-            ('contact', ListField()),
+            ('contact', StringField()),
             ('author', ListField()),
+            ('author_file', FileTextField()),
 
             ('vcs_tool', SingleLineField()),
             ('vcs_repository', SingleLineField()),
@@ -701,6 +749,7 @@ class About(object):
 
             ('checksum_md5', SingleLineField()),
             ('checksum_sha1', SingleLineField()),
+            ('checksum_sha256', SingleLineField()),
             ('spec_version', SingleLineField()),
         ])
 
@@ -710,7 +759,7 @@ class About(object):
             field.name = name
             setattr(self, name, field)
 
-    def __init__(self, location=None, about_file_path=None, use_mapping=False):
+    def __init__(self, location=None, about_file_path=None, use_mapping=False, mapping_file=None):
         self.create_fields()
         self.custom_fields = OrderedDict()
 
@@ -724,7 +773,7 @@ class About(object):
         self.base_dir = None
         if self.location:
             self.base_dir = os.path.dirname(location)
-            self.load(location, use_mapping)
+            self.load(location, use_mapping, mapping_file)
 
     def __repr__(self):
         return repr(self.all_fields())
@@ -744,7 +793,7 @@ class About(object):
         """
         attrib_fields = ['name',
                          'version',
-                         'license',
+                         'license_key',
                          'license_name',
                          'license_file',
                          'license_url',
@@ -753,7 +802,7 @@ class About(object):
                          'notice_url',
                          'redistribute',
                          'attribute',
-                         'track_change',
+                         'track_changes',
                          'modified',
                          'changelog_file',
                          'owner',
@@ -784,6 +833,7 @@ class About(object):
         If with_empty, include empty fields.
         """
         all_fields = []
+        bool_fields = ['redistribute', 'attribute', 'track_changes', 'modified']
         for field in list(self.fields.values()) + list(self.custom_fields.values()):
             if field.required:
                 all_fields.append(field)
@@ -796,7 +846,10 @@ class About(object):
                         all_fields.append(field)
                     elif field.present and field.value:
                         all_fields.append(field)
-
+                    elif field.name in bool_fields:
+                        # Check is the value valid
+                        if not field.value == None:
+                            all_fields.append(field)
                 else:
                     if field.present:
                         if not field.value:
@@ -821,28 +874,13 @@ class About(object):
         if with_paths:
             afpa = self.about_file_path_attr
             as_dict[afpa] = self.about_file_path
-            arpa = self.about_resource_path_attr
-            if self.about_resource_path.present:
-                as_dict[arpa] = self.resolved_resources_paths()
-            else:
-                # Create a relative 'about_resource_path' if user has not defined
-                if self.about_resource.present:
-                    for resource_name in self.about_resource.value:
-                        if resource_name == '.':
-                            as_dict[arpa] = '.'
-                        else:
-                            as_dict[arpa] = './' + resource_name
-                # Return an empty 'about_resource_path' if the 'about_resource'
-                # key is not found
-                else:
-                    as_dict[arpa] = ''
 
         for field in self.all_fields(with_absent=with_absent,
                                      with_empty=with_empty):
             as_dict[field.name] = field.serialized_value()
         return as_dict
 
-    def hydrate(self, fields, use_mapping=False):
+    def hydrate(self, fields, use_mapping=False, mapping_file=None):
         """
         Process an iterable of field (name, value) tuples. Update or create
         Fields attributes and the fields and custom fields dictionaries.
@@ -852,9 +890,8 @@ class About(object):
         seen_fields = OrderedDict()
 
         mapping = {}
-        if use_mapping:
-            mapping = util.get_mapping()
-
+        if use_mapping or mapping_file:
+            mapping = util.get_mapping(mapping_file)
         for name, value in fields:
             orig_name = name
             name = name.lower()
@@ -879,7 +916,7 @@ class About(object):
                 standard_field.present = True
                 continue
 
-            if not use_mapping:
+            if not use_mapping and not mapping_file:
                 if not name == self.about_file_path_attr:
                     msg = (u'Field %(orig_name)s is not a supported field and is ignored.')
                     errors.append(Error(INFO, msg % locals()))
@@ -935,7 +972,8 @@ class About(object):
         return errors
 
     def process(self, fields, about_file_path, running_inventory=False,
-                base_dir=None, license_notice_text_location=None, use_mapping=False):
+                base_dir=None, license_notice_text_location=None,
+                use_mapping=False, mapping_file=None):
         """
         Hydrate and validate a sequence of field name/value tuples from an
         ABOUT file. Return a list of errors.
@@ -944,14 +982,13 @@ class About(object):
         self.license_notice_text_location = license_notice_text_location
         afp = self.about_file_path
         errors = []
-        hydratation_errors = self.hydrate(fields, use_mapping=use_mapping)
+        hydratation_errors = self.hydrate(fields, use_mapping=use_mapping, mapping_file=mapping_file)
         errors.extend(hydratation_errors)
 
         # We want to copy the license_files before the validation
         if license_notice_text_location:
             copy_license_notice_files(
                 fields, base_dir, license_notice_text_location, afp)
-
         # we validate all fields, not only these hydrated
         all_fields = self.all_fields()
         validation_errors = validate_fields(
@@ -965,7 +1002,7 @@ class About(object):
         # self.about_resource.resolve(self.about_file_path)
         return errors
 
-    def load(self, location, use_mapping=False):
+    def load(self, location, use_mapping=False, mapping_file=None):
         """
         Read, parse and process the ABOUT file at location.
         Return a list of errors and update self with errors.
@@ -980,15 +1017,20 @@ class About(object):
                 input_text = txt.read()
             """
             The running_inventory defines if the current process is 'inventory' or not.
-            This is used for the validation of the about_resource_path.
+            This is used for the validation of the path of the 'about_resource'.
             In the 'inventory' command, the code will use the parent of the about_file_path
-            location and join with the 'about_resource_path' for the validation.
+            location and join with the 'about_resource' for the validation.
             On the other hand, in the 'gen' command, the code will use the
             generated location (aka base_dir) along with the parent of the about_file_path
-            and then join with the 'about_resource_path'
+            and then join with the 'about_resource'
             """
             running_inventory = True
-            errs = self.load_dict(saneyaml.load(input_text), base_dir, running_inventory, use_mapping)
+            # wrap the value of the boolean field in quote to avoid
+            # automatically conversion from yaml.load
+            input = util.wrap_boolean_value(input_text)  # NOQA
+            data = saneyaml.load(input, allow_duplicate_keys=False)
+
+            errs = self.load_dict(data, base_dir, running_inventory, use_mapping, mapping_file)
             errors.extend(errs)
         except Exception as e:
             msg = 'Cannot load invalid ABOUT file: %(location)r: %(e)r\n' + str(e)
@@ -997,28 +1039,8 @@ class About(object):
         self.errors = errors
         return errors
 
-    def loads(self, string, base_dir):
-        """
-        Load the ABOUT file from string. Return a list of errors.
-        """
-        lines = string.splitlines(True)
-        errors = self.load_lines(lines, base_dir)
-        self.errors = errors
-        return errors
-
-    def load_lines(self, lines, base_dir):
-        """
-        Load the ABOUT file from a lines list. Return a list of errors.
-        """
-        errors = []
-        parse_errors, fields = parse(lines)
-        errors.extend(parse_errors)
-        process_errors = self.process(fields, base_dir)
-        errors.extend(process_errors)
-        self.errors = errors
-        return errors
-
-    def load_dict(self, fields_dict, base_dir, running_inventory=False, use_mapping=False,
+    def load_dict(self, fields_dict, base_dir, running_inventory=False,
+                  use_mapping=False, mapping_file=None,
                   license_notice_text_location=None, with_empty=True):
         """
         Load the ABOUT file from a fields name/value mapping.
@@ -1026,29 +1048,83 @@ class About(object):
         Return a list of
         errors.
         """
-        fields = fields_dict.items()
+        fields = list(fields_dict.items())
         about_file_path = self.about_file_path
         if not with_empty:
             fields = [(n, v) for n, v in fields_dict.items() if v]
+        for key, value in fields:
+            if key == u'licenses':
+                lic_key, lic_name, lic_file, lic_url = ungroup_licenses(value)
+                if lic_key:
+                    fields.append(('license_key', lic_key))
+                if lic_name:
+                    fields.append(('license_name', lic_name))
+                if lic_file:
+                    fields.append(('license_file', lic_file))
+                if lic_url:
+                    fields.append(('license_url', lic_url))
+                # The licenses field has been ungrouped and can be removed.
+                # Otherwise, it will gives the following INFO level error
+                # 'Field licenses is not a supported field and is ignored.'
+                licenses_field = (key, value)
+                fields.remove(licenses_field)
         errors = self.process(
-            fields, about_file_path, running_inventory, base_dir, 
-            license_notice_text_location, use_mapping)
+            fields, about_file_path, running_inventory, base_dir,
+            license_notice_text_location, use_mapping, mapping_file)
         self.errors = errors
         return errors
 
-    def dumps(self, with_absent=False, with_empty=True):
+
+    def dumps(self, use_mapping=False, mapping_file=False, with_absent=False, with_empty=True):
         """
         Return self as a formatted ABOUT string.
         If with_absent, include absent (not present) fields.
         If with_empty, include empty fields.
         """
-        serialized = []
+        about_data = OrderedDict()
+        # Group the same license information (name, url, file) together
+        license_key = []
+        license_name = []
+        license_file = []
+        license_url = []
+        file_fields = ['about_resource', 'notice_file', 'changelog_file', 'author_file']
+        bool_fields = ['redistribute', 'attribute', 'track_changes', 'modified']
         for field in self.all_fields(with_absent, with_empty):
-            serialized.append(field.serialize())
-        # always end with a new line
-        return u'\n'.join(serialized) + u'\n'
+            if field.name == 'license_key' and field.value:
+                license_key = field.value
+            elif field.name == 'license_name' and field.value:
+                license_name = field.value
+            elif field.name == 'license_file' and field.value:
+                license_file = field.value.keys()
+            elif field.name == 'license_url' and field.value:
+                license_url = field.value
+            # No multiple 'about_resource' reference supported.
+            # Take the first element (should only be one) in the list for the
+            # value of 'about_resource'
+            elif field.name in file_fields and field.value:
+                about_data[field.name] = list(field.value.keys())[0]
+            else:
+                if field.value or (field.name in bool_fields and not field.value == None):
+                    about_data[field.name] = field.value
 
-    def dump(self, location, with_absent=False, with_empty=True):
+        # Group the same license information in a list
+        license_group = list(zip_longest(license_key, license_name, license_file, license_url))
+        for lic_group in license_group:
+            lic_dict = OrderedDict()
+            if lic_group[0]:
+                lic_dict['key'] = lic_group[0]
+            if lic_group[1]:
+                lic_dict['name'] = lic_group[1]
+            if lic_group[2]:
+                lic_dict['file'] = lic_group[2]
+            if lic_group[3]:
+                lic_dict['url'] = lic_group[3]
+            about_data.setdefault('licenses', []).append(lic_dict)
+        formatted_about_data = util.format_output(about_data, use_mapping, mapping_file)
+
+        return saneyaml.dump(formatted_about_data, indent=2)
+
+    def dump(self, location, use_mapping=False, mapping_file=False, with_absent=False, with_empty=True):
         """
         Write formatted ABOUT representation of self to location.
         If with_absent, include absent (not present) fields.
@@ -1068,7 +1144,7 @@ class About(object):
         if on_windows:
             about_file_path = add_unc(about_file_path)
         with codecs.open(about_file_path, mode='wb', encoding='utf-8') as dumped:
-            dumped.write(self.dumps(with_absent, with_empty))
+            dumped.write(self.dumps(use_mapping, mapping_file, with_absent, with_empty))
 
     def dump_lic(self, location, license_dict):
         """
@@ -1085,7 +1161,8 @@ class About(object):
 
         if self.license_expression.present and not self.license_file.present:
             special_char_in_expression, lic_list = parse_license_expression(self.license_expression.value)
-            self.license = lic_list
+            self.license_key.value = lic_list
+            self.license_key.present = True
             if not special_char_in_expression:
                 for lic_key in lic_list:
                     try:
@@ -1111,7 +1188,7 @@ valid_field_name = re.compile(field_name, re.UNICODE | re.IGNORECASE).match
 # line in the form of "name: value"
 field_declaration = re.compile(
     r'^'
-    + field_name +
+    +field_name +
     r'\s*:\s*'
     r'(?P<value>.*)'
     r'\s*$'
@@ -1190,7 +1267,7 @@ def parse(lines):
     return errors, fields
 
 
-def collect_inventory(location, use_mapping=False):
+def collect_inventory(location, use_mapping=False, mapping_file=None):
     """
     Collect ABOUT files at location and return a list of errors and a list of
     About objects.
@@ -1205,7 +1282,7 @@ def collect_inventory(location, use_mapping=False):
     abouts = []
     for about_loc in about_locations:
         about_file_path = util.get_relative_path(input_location, about_loc)
-        about = About(about_loc, about_file_path, use_mapping)
+        about = About(about_loc, about_file_path, use_mapping, mapping_file)
         # Insert about_file_path reference to the error
         for severity, message in about.errors:
             msg = (about_file_path + ": " + message)
@@ -1216,7 +1293,6 @@ def collect_inventory(location, use_mapping=False):
     for about_error in errors:
         if not about_error in dedup_errors:
             dedup_errors.append(about_error)
-
     return dedup_errors, abouts
 
 
@@ -1280,25 +1356,32 @@ def about_object_to_list_of_dictionary(abouts, with_absent=False, with_empty=Tru
             afp = ad['about_file_path']
             afp = '/' + afp if not afp.startswith('/') else afp
             ad['about_file_path'] = afp
-        if not 'about_resource_path' in ad.keys():
-            arp = ad['about_resource_path']
         abouts_dictionary_list.append(ad)
     return abouts_dictionary_list
 
 
-def write_output(abouts, location, format, with_absent=False, with_empty=True):
+def write_output(abouts, location, format, mapping_output=None, with_absent=False, with_empty=True):  # NOQA
     """
     Write a CSV/JSON file at location given a list of About objects
     """
     errors = []
     about_dictionary_list = about_object_to_list_of_dictionary(abouts, with_absent, with_empty)
+    if mapping_output:
+        updated_dictionary_list = util.update_about_dictionary_keys(about_dictionary_list, mapping_output)
+    else:
+        updated_dictionary_list = about_dictionary_list
     location = add_unc(location)
     with codecs.open(location, mode='wb', encoding='utf-8') as output_file:
         if format == 'csv':
             fieldnames = field_names(abouts)
-            writer = csv.DictWriter(output_file, fieldnames)
+            if mapping_output:
+                updated_fieldnames = util.update_fieldnames(fieldnames, mapping_output)
+            else:
+                updated_fieldnames = fieldnames
+            writer = csv.DictWriter(output_file, updated_fieldnames)
             writer.writeheader()
-            for row in about_dictionary_list:
+            csv_formatted_list = util.format_about_dict_for_csv_output(updated_dictionary_list)
+            for row in csv_formatted_list:
                 # See https://github.com/dejacode/about-code-tool/issues/167
                 try:
                     writer.writerow(row)
@@ -1306,7 +1389,8 @@ def write_output(abouts, location, format, with_absent=False, with_empty=True):
                     msg = u'Generation skipped for ' + row['about_file_path'] + u' : ' + str(e)
                     errors.append(Error(CRITICAL, msg))
         else:
-            output_file.write(json.dumps(about_dictionary_list, indent=2))
+            json_fomatted_list = util.format_about_dict_for_json_output(updated_dictionary_list)
+            output_file.write(json.dumps(json_fomatted_list, indent=2))
     return errors
 
 
@@ -1372,8 +1456,8 @@ def by_license_content(abouts):
     grouped[''] = []
     no_license = grouped['']
     for about in abouts:
-        if about.license.value:
-            special_char_in_expression, lic_list = parse_license_expression(about.license.value)
+        if about.license_key.value:
+            special_char_in_expression, lic_list = parse_license_expression(about.license_key.value)
             if not special_char_in_expression:
                 for lic in lic_list:
                     if lic in grouped:
@@ -1445,9 +1529,8 @@ def parse_license_expression(lic_expression):
 
 def special_char_in_license_expresion(lic_expression):
     not_support_char = [
-        '!', '@', '#', '$', '%', '^', '&', '*', '(', ')',
-        '+', '=', '{', '}', '|', '[', ']', '\\',
-        ':', ';', '<', '>', '?', ',', '/']
+        '!', '@', '#', '$', '%', '^', '&', '*', '=', '{', '}',
+        '|', '[', ']', '\\', ':', ';', '<', '>', '?', ',', '/']
     special_character = []
     for char in not_support_char:
         if char in lic_expression:
@@ -1498,45 +1581,3 @@ def verify_license_files_in_location(about, lic_location):
     return license_location_dict, errors
 
 
-def check_file_field_exist(about, location):
-    """
-    Return a list of errors for non-existence file in file fields
-    """
-    errors = []
-    loc = util.to_posix(location)
-    parent = posixpath.dirname(loc)
-
-    about_file_path = util.to_posix(os.path.join(parent, os.path.basename(parent)))
-
-    # The model only has the following as FileTextField
-    license_files = about.license_file.value
-    notice_files = about.notice_file.value
-    changelog_files = about.changelog_file.value
-
-    if license_files:
-        for lic in license_files:
-            lic_path = posixpath.join(dirname(util.to_posix(about_file_path)), lic)
-            if not posixpath.exists(lic_path):
-                msg = (u'Field license_file: Path '
-                   u'%(lic_path)s '
-                   u'not found' % locals())
-                errors.append(msg)
-
-    if notice_files:
-        for notice in notice_files:
-            notice_path = posixpath.join(dirname(util.to_posix(about_file_path)), notice)
-            if not posixpath.exists(notice_path):
-                msg = (u'Field notice_file: Path '
-                   u'%(notice_path)s '
-                   u'not found' % locals())
-                errors.append(msg)
-
-    if changelog_files:
-        for changelog in changelog_files:
-            changelog_path = posixpath.join(dirname(util.to_posix(about_file_path)), changelog)
-            if not posixpath.exists(changelog_path):
-                msg = (u'Field changelog_file: Path '
-                   u'%(changelog_path)s '
-                   u'not found' % locals())
-                errors.append(msg)
-    return errors
